@@ -1,14 +1,17 @@
+#![no_std] // This library doesn't require std, but its dependencies do
+extern crate alloc;
 use thread_local::ThreadLocal;
-use std::cell::{RefCell};
+use core::cell::RefCell;
 use arc_swap::ArcSwap;
-use std::sync::Arc;
+use alloc::sync::Arc;
+use owning_ref::ArcRef;
 
-pub struct Parameter<T: Clone + Send> {
+pub struct Parameter<T: Send + Sync> {
     base: ArcSwap<T>,
-    override_value: ThreadLocal<RefCell<Option<T>>>,
+    override_value: ThreadLocal<RefCell<Option<Arc<T>>>>,
 }
 
-impl <T: Clone + Send> Parameter<T> {
+impl<T: Send + Sync> Parameter<T> {
     pub fn new(x: T) -> Parameter<T> {
         Parameter { base: ArcSwap::from(Arc::new(x)), override_value: ThreadLocal::new() }
     }
@@ -16,18 +19,20 @@ impl <T: Clone + Send> Parameter<T> {
         self.base.store(Arc::new(x));
     }
     pub fn with<U, F>(&self, x: T, f: F) -> U where F: FnOnce() -> U {
-        let origin = self.override_value.get_or(|| RefCell::new(None)).replace(Some(x));
+        let origin = self.override_value.get_or(|| RefCell::new(None)).replace(Some(Arc::new(x)));
         let result = f();
-        let cell = self.override_value.get().unwrap();
-        cell.replace(origin);
+        self.override_value.get().unwrap().replace(origin);
         result
     }
-    pub fn get_clone(&self) -> T {
+    fn get_arc(&self) -> Arc<T> {
         if let Some(cell) = self.override_value.get() {
             if let Some(override_value) = &*cell.borrow() {
                 return override_value.clone();
             }
         }
-        (**self.base.load()).clone()
+        (*self.base.load()).clone()
+    }
+    pub fn get(&self) -> ArcRef<T> {
+        ArcRef::new(self.get_arc())
     }
 }
